# $Id: connection.py 15 2004-05-17 02:55:58Z Jean-François Doyon $

""" Subversion connection class. """

import os
from apelib.fs.connection import FSConnection
from svn import repos, fs, util
from fileops import SubversionFileOperations
from AccessControl import getSecurityManager
from annotated import PropertyFilesystem

DEBUG = os.environ.get('CAESAR_DEBUG')

class SubversionConnection(FSConnection):

    def __init__(self, repository, basepath):
        util.apr_initialize()
        FSConnection.__init__(self, basepath, ops=SubversionFileOperations())
        self.afs = PropertyFilesystem(self.ops, '.', '_')
        self.secman = getSecurityManager()
        # This pool never gets cleaned out since it needs to exist for the
        # life of the connection, and the connection never closes once it's
        # open so far as I can tell (Not until shutdown that is)
        pool = util.svn_pool_create(None)
        r = repos.svn_repos_find_root_path(repository, pool)
        if not r: raise IOError("Bad repository path: %s" % repository)
        else: self.ops.repos = repos.svn_repos_open(r, pool)

    # IFSReader overloads

    def read_mod_time(self, oid, default=0):
        path = self.get_path(oid)
        if DEBUG: print "got read_mod_time for path '%s'" % path
        time = -1
        try:
            time = self.ops.getmtime(path)
        except OSError:
            pass
        if ( time == -1 ): time = default
        return time

    def get_sources(self, oid):
        path = self.get_path(oid)
        if DEBUG: print "got get_sources for path '%s'" % path
        time = []
        try:
            time.append(self.ops.getmtime(path))
        except OSError:
            time.append(None)
        return {(self, (path,)): time}

    # IFSWriter overloads

    def _move_item(self, src, dest):
        ops = self.ops
        parent = ops.dirname(dest)
        if not ops.exists(parent):
            ops.makedirs(parent)
        ops.rename(src, dest)

    def _do_delete(self, oid):
        ops = self.ops
        path = self.get_path(oid)
        if path == self.basepath:
            # Delete the contents of the base directory.
            for fn in ops.listdir(path):
                if not fn in ('_root', '_tmp'):
                    self._rmrf(ops.join(self.basepath, fn))
        else:
            # Delete an object.
            self._rmrf(path)
        if self._tmp_subpaths.has_key(oid):
            del self._tmp_subpaths[oid]
        parents = self.table.get_parents(oid)
        for parent_oid, filename in parents:
            self.table.remove(parent_oid, filename)
            # XXX Need to garbage collect descendants in the OID table.

    # ITPCConnection overloads

    def connect(self):
        if DEBUG: print "got TPC connect"
        ops = self.ops
        # In order to do this we need to be in a "transactional"
        # state ...
        self.beginSVNTransaction()
        if not ops.exists(self.basepath):
            ops.makedirs(self.basepath)
        if self.app_filename:
            # If there are objects at basepath, create a _root
            # containing an application also.
            contents = self.afs.compute_contents(self.basepath)
            fn_to_name, name_to_fn = contents
            if fn_to_name:
                self._do_link_app(1)
        self.finishSVNTransaction()

    def finish(self):
        if DEBUG: print "got TPC finish"
        if self._final:
            self.beginSVNTransaction()
            try:
                for code in self._script:
                    m = getattr(self, '_do_%s' % code[0])
                    m(*code[1:])
            finally:
                self.finishSVNTransaction()

    # Subversion specific methods.

    def beginSVNTransaction(self):
        if DEBUG: print "beginning svn transaction"
        self.ops.txnpool = self.ops.getPool()
        revfs = repos.svn_repos_fs(self.ops.repos)
        self.ops.txnbaserevnum = fs.youngest_rev(revfs, self.ops.txnpool)
        self.ops.txnfs = repos.svn_repos_fs_begin_txn_for_commit(self.ops.repos, self.ops.txnbaserevnum, self.secman.getUser().getId(), '', self.ops.txnpool)
        self.ops.txnroot = fs.txn_root(self.ops.txnfs, self.ops.txnpool)
        self.ops.intransaction = True
        
    def finishSVNTransaction(self):
        if DEBUG: print "finishing svn transaction"
        repos.svn_repos_fs_commit_txn(self.ops.repos, self.ops.txnfs, self.ops.txnpool)
        self.ops.destroyPool(self.ops.txnpool)
        if hasattr(self.ops, 'txnfs'): del self.ops.txnfs
        if hasattr(self.ops, 'txnroot'): del self.ops.txnroot
        if hasattr(self.ops, 'txnbaserevnum'): del self.ops.txnbaserevnum
        if hasattr(self.ops, 'txnpool'): del self.ops.txnpool
        self.ops.intransaction = False
        self.reset()
