# $Id: fileops.py 15 2004-05-17 02:55:58Z Jean-François Doyon $

""" File operations for use by the subversion connection. """

import os
import posixpath
from svn import util, fs, repos

DEBUG = os.environ.get('CAESAR_DEBUG')

class SubversionFileOperations:

    def __init__(self):
        # Make sure we use POSIX paths with subversion,
        # and not the platform's version
        self.dirname = posixpath.dirname
        self.join = posixpath.join
        self.split = posixpath.split
        self.splitext = posixpath.splitext
        self.intransaction = False

    def getPool(self):
        return util.svn_pool_create(None)

    def destroyPool(self, pool):
        util.svn_pool_destroy(pool)

    def getRoot(self, pool):
        # Return the root to use.
        # Any operation that writes should get a txn root ...
        # It's up to the connection to make sure this occurs,
        # since it has the TPC interface.
        if self.intransaction:
            return self.txnroot
        else:
            # Not in a txn, use the youngest revision.
            svnfs = repos.svn_repos_fs(self.repos)
            revnum = fs.youngest_rev(svnfs, pool)
            root = fs.revision_root(svnfs, revnum, pool)
            return root

    def isdir(self, path):
        if DEBUG: print "got isdir for path '%s'" % path
        pool = self.getPool()
        data = fs.is_dir(self.getRoot(pool), path, pool)
        self.destroyPool(pool)
        return data

    def listdir(self, path):
        if DEBUG: print "got listdir for path '%s'" % path
        pool = self.getPool()
        data = fs.dir_entries(self.getRoot(pool), path, pool)
        self.destroyPool(pool)
        return data.keys()

    def mkdir(self, path):
        if DEBUG: print "got mkdir for path '%s'" % path
        pool = self.getPool()
        fs.make_dir(self.getRoot(pool), path, pool)
        self.destroyPool(pool)

    def remove(self, path):
        if DEBUG: print "got remove for path '%s'" % path
        pool = self.getPool()
        fs.delete(self.getRoot(pool), path, pool)
        self.destroyPool(pool)

    rmtree = remove # svn's 'remove' handles both files and dirs and is recursive

    def makedirs(self, path):
        if DEBUG: print "got mkdirs for path '%s'" % path
        head, tail = posixpath.split(path)
        if not tail:
            head, tail = posixpath.split(head)
        if head and tail and not self.exists(head):
            self.makedirs(head)
        self.mkdir(path)

    def exists(self, path):
        if DEBUG: print "got exists for path '%s'" % path
        pool = self.getPool()
        r = fs.check_path(self.getRoot(pool), path, pool)
        self.destroyPool(pool)
        if r == util.svn_node_none: return False
        else: return True

    def getmtime(self, path):
        if DEBUG: print "got getmtime for path '%s'" % path
        pool = self.getPool()
        try:
            revnum, date, author = repos.svn_repos_get_committed_info(self.getRoot(pool), path, pool)
        except:
            raise OSError
        fromepochinmicro = util.svn_time_from_cstring(date, pool)
        self.destroyPool(pool)
        return fromepochinmicro/1000000

    def rename(self, src, dst):
        if DEBUG: print "got rename from '%s' to '%s'" % (src, dst)
        pool = self.getPool()
        txnroot = self.getRoot(pool)
        revfs = repos.svn_repos_fs(self.repos)
        revroot = fs.revision_root(revfs, self.txnbaserevnum, pool)
        fs.copy(revroot, src, txnroot, dst, pool);
        fs.delete(txnroot, src, pool)
        self.destroyPool(pool)

    def readfile(self, path, as_text):
        if DEBUG: print "got readfile for path '%s'" % path
        pool = self.getPool()
        root = self.getRoot(pool)
        if not self.exists(path):
            raise IOError("File %s does not exist" % path )
        stream = fs.file_contents(root, path, pool)
        data = util.svn_stream_read(stream, int(fs.file_length(root, path, pool)))
        util.svn_stream_close(stream)
        self.destroyPool(pool)
        return data

    def readproperty(self, path, name):
        if DEBUG: print "got readproperty for path '%s', property '%s'" % (path, name)
        pool = self.getPool()
        data = fs.node_prop(self.getRoot(pool), path, name, pool)
        self.destroyPool(pool)
        return data
    
    def writefile(self, path, as_text, bytes):
        if DEBUG: print "got writefile for path '%s'" % path
        pool = self.getPool()
        root = self.getRoot(pool)
        if not self.exists(path):
            fs.make_file(root, path, pool)
        stream = fs.apply_text(root, path, None, pool)
        util.svn_stream_write(stream, bytes)
        util.svn_stream_close(stream)
        self.destroyPool(pool)

    def writeproperty(self, path, name, data):
        if DEBUG: print "got writeproperty for path '%s', property '%s', data '%s'" % (path, name, data)
        pool = self.getPool()
        return fs.change_node_prop(self.getRoot(pool), path, name, data, pool)
        self.destroyPool(pool)

    def canwrite(self, path):
        return 1
